@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-from pyrf.devices.thinkrf import WSA4000
+from pyrf.devices.thinkrf import WSA
 from pyrf.connectors.twisted_async import TwistedConnector
 from pyrf.sweep_device import SweepDevice
 
@@ -32,11 +32,11 @@ def plot_sweep(fstart, fstop, bins):
 def start_sweep(v):
     global sd
     sd = SweepDevice(dut, plot_sweep)
-    sd.capture_power_spectrum(0e9, 10e9, 2000,
+    sd.capture_power_spectrum(0e9, 10e9, 5e6,
         {'gain': 'high', 'antenna': 1})
 
 # connect to wsa
-dut = WSA4000(connector=TwistedConnector(reactor))
+dut = WSA(connector=TwistedConnector(reactor))
 d = dut.connect(sys.argv[1])
 d.addCallbacks(start_sweep, twisted.python.log.err)
 reactor.run()
