@@ -21,7 +21,7 @@ import constants
 import control_util as cu
 from plot_widget import plot
 import gui_config
-from pyrf.devices.thinkrf import WSA4000
+from pyrf.devices.thinkrf import WSA
 from pyrf.sweep_device import SweepDevice
 from pyrf.connectors.twisted_async import TwistedConnector
 from pyrf.config import TriggerSettings
@@ -52,8 +52,7 @@ class MainWindow(QtGui.QMainWindow):
         name = None
         if len(sys.argv) > 1:
             name = sys.argv[1]
-        self.mainPanel = MainPanel(name)
-        self.setCentralWidget(self.mainPanel)
+        self.mainPanel = MainPanel()
         openAction = QtGui.QAction('&Open Device', self)
         openAction.triggered.connect( self.mainPanel.open_device_dialog)
         exitAction = QtGui.QAction('&Exit', self)
@@ -64,8 +63,13 @@ class MainWindow(QtGui.QMainWindow):
         fileMenu.addAction(openAction)
         fileMenu.addAction(exitAction)
         self.setWindowTitle('Spectrum Analyzer')
-
-
+        self.setCentralWidget(self.mainPanel)
+        self.mainPanel.show()
+        if name:
+            self.mainPanel.open_device(name)
+        else:
+            self.mainPanel.open_device_dialog()
+    
     def closeEvent(self, event):
         if self.mainPanel.dut:
             self.mainPanel.dut.abort()
@@ -79,12 +83,12 @@ class MainPanel(QtGui.QWidget):
     """
     The spectrum view and controls
     """
-    def __init__(self, name):
+    def __init__(self):
         self.dut = None
         self.control_widgets = []
         super(MainPanel, self).__init__()
         self.setMinimumWidth(800)
-        self.setMinimumHeight(400)
+        self.setMinimumHeight(600)
         self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
         self.plot_state = gui_config.plot_state()
         # plot window
@@ -93,11 +97,7 @@ class MainPanel(QtGui.QWidget):
         self.initUI()
         self.disable_controls()
         self._reactor = self._get_reactor()
-        if name:
-            self.open_device(name)
-        else:
-            self.open_device_dialog()
-               
+
     def _get_reactor(self):
         # late import because installReactor is being used
         from twisted.internet import reactor
@@ -119,12 +119,24 @@ class MainPanel(QtGui.QWidget):
 
     @inlineCallbacks
     def open_device(self, name):
-        dut = WSA4000(connector=TwistedConnector(self._reactor))
+        dut = WSA(connector=TwistedConnector(self._reactor))
         yield dut.connect(name)
         self.dut = dut
+        if dut.properties.model == 'WSA5000':
+            self._antenna_box.hide()
+            self._gain_box.hide()
+            self._trigger.hide()
+            #self._attenuator_box.show()
+        else:
+            self._antenna_box.show()
+            self._gain_box.show()
+            self._trigger.show()
+            #self._attenuator_box.hide()
+
         self.sweep_dut = SweepDevice(dut, self.receive_data)
         self.cap_dut = CaptureDevice(dut, self.receive_data)
         self.enable_controls()
+        cu._select_fstart(self)
         self.read_sweep()
 
     def read_sweep(self):
@@ -133,7 +145,7 @@ class MainPanel(QtGui.QWidget):
         self.plot_state.dev_set.pop('trigger', None)
         self.sweep_dut.capture_power_spectrum(self.plot_state.fstart,
                                               self.plot_state.fstop,
-                                              self.plot_state.bin_size,
+                                              self.plot_state.rbw,
                                               self.plot_state.dev_set,
                                               continuous = False)
 
@@ -143,7 +155,7 @@ class MainPanel(QtGui.QWidget):
         device_set['freq'] = self.plot_state.center_freq
         device_set['trigger'] = self.plot_state.trig_set
 
-        self.cap_dut.capture_power_spectrum(device_set,self.plot_state.bin_size)
+        self.cap_dut.capture_power_spectrum(device_set,self.plot_state.rbw)
 
 
     def receive_data(self, fstart, fstop, pow_):
@@ -188,156 +200,50 @@ class MainPanel(QtGui.QWidget):
         grid.setSpacing(10)
         for x in range(8):
             grid.setColumnMinimumWidth(x, 300)
-        grid.setRowMinimumHeight(10, 800)
+        grid.setRowMinimumHeight(14, 800)
 
         # add plot widget
         plot_width = 8
-        grid.addWidget(self._plot.window,1,0,10,plot_width)
+        grid.addWidget(self._plot.window,0,0,15,plot_width)
 
         marker_label, delta_label, diff_label = self._marker_labels()
-        grid.addWidget(marker_label, 1, 1, 1, 2)
-        grid.addWidget(delta_label, 1, 3, 1, 2)
-        grid.addWidget(diff_label , 1, 5, 1, 2)
-        
-        x = 0    
+        grid.addWidget(marker_label, 0, 1, 1, 2)
+        grid.addWidget(delta_label, 0, 3, 1, 2)
+        grid.addWidget(diff_label , 0, 5, 1, 2)
+ 
         y = 0
-        trig = self._trigger_control()
-        grid.addWidget(trig, y, x, 1, 1)
-        mark = self._marker_control()
-        grid.addWidget(mark, y, x + 1, 1, 1)
-        delta = self._delta_control()
-        grid.addWidget(delta, y, x + 2, 1, 1)
-        mhold = self._mhold_control()
-        grid.addWidget(mhold, y, x + 3, 1, 1)
-        
-        x = 4
-        pause = self._pause_control()
-        grid.addWidget(pause, y, x, 1, 1)
-        peak = self._peak_control()
-        grid.addWidget(peak, y, x + 1, 1, 1)
-        grid_en = self._grid_control()
-        grid.addWidget(grid_en, y, x + 2, 1, 1)
-        cu._grid_control(self)
-        center = self._center_control()
-        grid.addWidget(center, y, x + 3, 1, 1)
-        
         x = plot_width
-        grid.addWidget(self._antenna_control(), y, x, 1, 2)
-        
-        x = plot_width 
-        y += 1
-        grid.addWidget(self._gain_control(), y, x, 1, 2)
-        grid.addWidget(QtGui.QLabel('IF Gain:'), y, x + 2, 1, 1)
-        grid.addWidget(self._ifgain_control(), y, x + 3, 1, 1)
-        
-        x = plot_width
-        y += 1
-        fstart_bt, fstart_txt = self._fstart_controls()
-        grid.addWidget(fstart_bt, y, x, 1, 1)
-        grid.addWidget(fstart_txt, y, x + 1, 1, 2)
-        grid.addWidget(QtGui.QLabel('MHz'), y, x + 3, 1, 1)
-        
-        x = plot_width
-        y += 1
-        cfreq, freq, steps, freq_plus, freq_minus = self._freq_controls()
-        grid.addWidget(cfreq, y, x, 1, 1)
-        grid.addWidget(freq, y, x + 1, 1, 2)
-        grid.addWidget(QtGui.QLabel('MHz'), y, x + 3, 1, 1)
-        x = plot_width
-        y += 1
-        bw_bt, bw_txt = self._bw_controls()
-        grid.addWidget(bw_bt, y, x, 1, 1)
-        grid.addWidget(bw_txt, y, x + 1, 1, 2)
-        grid.addWidget(QtGui.QLabel('MHz'), y, x + 3, 1, 1)
-        
-        x = plot_width
-        y += 1
-        fstop_bt, fstop_txt = self._fstop_controls()
-        grid.addWidget(fstop_bt, y, x, 1, 1)
-        grid.addWidget(fstop_txt, y, x + 1, 1, 2)
-        grid.addWidget(QtGui.QLabel('MHz'), y, x + 3, 1, 1)
-        
-        x = plot_width
-        y += 1
-        grid.addWidget(freq_minus, y, x, 1, 1)
-        grid.addWidget(steps, y, x + 1, 1, 2)
-        grid.addWidget(freq_plus, y, x + 3, 1, 1)
-        
-        x = plot_width
-        y += 1
-        rbw = self._rbw_controls()
-        grid.addWidget(QtGui.QLabel('Resolution\nBandwidth:'), y, x, 1, 1)
-        grid.addWidget(rbw, y, x + 1, 1, 3)
-                    
-        cu._select_fstart(self)
+        grid.addWidget(self._device_controls(), y, x, 2, 5)
+        y += 2
+        grid.addWidget(self._freq_controls(), y, x, 4, 5)
+        y += 4
+        grid.addWidget(self._plot_controls(), y, x, 4, 5)
+
         self.update_freq()
         self.setLayout(grid)
-        self.show()
-    
-    def _trigger_control(self):
-        trigger = QtGui.QPushButton('Trigger', self)
-        trigger.setToolTip("[T]\nTurn the Triggers on/off") 
-        trigger.clicked.connect(lambda: cu._trigger_control(self))
-        self._trigger = trigger
-        self.control_widgets.append(self._trigger)
-        return trigger
-    
-    def _marker_control(self):
-        marker = QtGui.QPushButton('Marker 1', self)
-        marker.setToolTip("[M]\nTurn Marker 1 on/off") 
-        marker.clicked.connect(lambda: cu._marker_control(self))
-        self._marker = marker
-        self.control_widgets.append(self._marker)
-        return marker
-        
-    def _delta_control(self):
-        delta = QtGui.QPushButton('Marker 2', self)
-        delta.setToolTip("[K]\nTurn Marker 2 on/off") 
-        delta.clicked.connect(lambda: cu._delta_control(self))
-        self._delta = delta
-        self.control_widgets.append(self._delta)
-        return delta
-    
-    def _peak_control(self):
-        peak = QtGui.QPushButton('Peak', self)
-        peak.setToolTip("[P]\nFind peak of the selected spectrum") 
-        peak.clicked.connect(lambda: cu._find_peak(self))
-        self._peak = peak
-        self.control_widgets.append(self._peak)
-        return peak
-        
-    def _mhold_control(self):
-        mhold = QtGui.QPushButton('Max Hold', self)
-        mhold.setToolTip("[H]\nTurn the Max Hold on/off") 
-        mhold.clicked.connect(lambda: cu._mhold_control(self))
-        self._mhold = mhold
-        self.control_widgets.append(self._mhold)
-        return mhold
-        
-    def _grid_control(self):
-        plot_grid = QtGui.QPushButton('Grid', self)
-        plot_grid.setToolTip("[G]\nTurn the Grid on/off") 
-        plot_grid.clicked.connect(lambda: cu._grid_control(self))
-        self._grid = plot_grid
-        self.control_widgets.append(self._grid)
-        return plot_grid
 
-    def _center_control(self):
-        center = QtGui.QPushButton('Recenter', self)
-        center.setToolTip("[C]\nCenter the Plot View around the available spectrum") 
-        center.clicked.connect(lambda: cu._center_plot_view(self))
-        self._center = center
-        self.control_widgets.append(self._center)
-        return center
         
-    def _pause_control(self):
-        pause = QtGui.QPushButton('Pause', self)
-        pause.setToolTip("[Space Bar]\n pause the plot window") 
-        pause.clicked.connect(lambda: cu._enable_plot(self))
-        self._pause = pause
-        self.control_widgets.append(self._pause)
-        return pause
-           
+    
+    def _device_controls(self):
+        dev_group = QtGui.QGroupBox("Device Control")
+        self.dev_group = dev_group
+        
+        dev_layout = QtGui.QVBoxLayout()
+        
+        first_row = QtGui.QHBoxLayout()
+        first_row.addWidget(self._antenna_control())
+        first_row.addWidget(self._trigger_control())
+        first_row.addWidget(self._attenuator_control())
+        
+        second_row = QtGui.QHBoxLayout()
+        second_row.addWidget(self._gain_control())
+        second_row.addWidget(self._ifgain_control())
+        
+        dev_layout.addLayout(first_row)
+        dev_layout.addLayout(second_row)
+
+        dev_group.setLayout(dev_layout)         
+        return dev_group
     def _antenna_control(self):
         antenna = QtGui.QComboBox(self)
         antenna.setToolTip("Choose Antenna") 
@@ -376,21 +282,93 @@ class MainPanel(QtGui.QWidget):
             self.plot_state.dev_set['ifgain'] = ifgain.value()
         ifgain.valueChanged.connect(new_ifgain)
         return ifgain
-            
+    
+    def _trigger_control(self):
+        trigger = QtGui.QCheckBox("Trigger")
+        trigger.setToolTip("[T]\nTurn the Triggers on/off") 
+        trigger.clicked.connect(lambda: cu._trigger_control(self))
+        self._trigger = trigger
+        self.control_widgets.append(self._trigger)
+        return trigger
+
+    def _attenuator_control(self):
+        attenuator = QtGui.QCheckBox("Attenuator")
+        attenuator.setChecked(True)
+        def new_attenuator():
+            self.plot_state.dev_set['attenuator'] = attenuator.isChecked()
+        attenuator.clicked.connect(new_attenuator)
+        self._attenuator_box = attenuator
+        self.control_widgets.append(attenuator)
+        return attenuator
+    
     def _freq_controls(self):
+        freq_group = QtGui.QGroupBox("Frequency Control")
+        self._freq_group = freq_group
+        
+        freq_layout = QtGui.QVBoxLayout()
+        
+        fstart_hbox = QtGui.QHBoxLayout()
+        fstart_bt, fstart_txt = self._fstart_controls()
+        fstart_hbox.addWidget(fstart_bt)
+        fstart_hbox.addWidget(fstart_txt)
+        fstart_hbox.addWidget(QtGui.QLabel('MHz'))
+        
+        cfreq_hbox = QtGui.QHBoxLayout()
+        cfreq_bt, cfreq_txt = self._center_freq()
+        cfreq_hbox.addWidget(cfreq_bt)
+        cfreq_hbox.addWidget(cfreq_txt)
+        cfreq_hbox.addWidget(QtGui.QLabel('MHz'))
+        
+        bw_hbox = QtGui.QHBoxLayout()
+        bw_bt, bw_txt = self._bw_controls()
+        bw_hbox.addWidget(bw_bt)
+        bw_hbox.addWidget(bw_txt)
+        bw_hbox.addWidget(QtGui.QLabel('MHz'))
+        
+        fstop_hbox = QtGui.QHBoxLayout()
+        fstop_bt, fstop_txt = self._fstop_controls()
+        fstop_hbox.addWidget(fstop_bt)
+        fstop_hbox.addWidget(fstop_txt)
+        fstop_hbox.addWidget(QtGui.QLabel('MHz'))
+        
+        freq_inc_hbox = QtGui.QHBoxLayout()
+        freq_inc_steps, freq_inc_minus, freq_inc_plus = self._freq_incr()
+        freq_inc_hbox.addWidget(freq_inc_minus)
+        freq_inc_hbox.addWidget(freq_inc_steps)
+        freq_inc_hbox.addWidget(freq_inc_plus)
+        
+        rbw_hbox = QtGui.QHBoxLayout()
+        rbw = self._rbw_controls()
+        rbw_hbox.addWidget(QtGui.QLabel('Resolution Bandwidth:'))
+        rbw_hbox.addWidget(rbw)
+        
+        freq_layout.addLayout(fstart_hbox)
+        freq_layout.addLayout(cfreq_hbox)
+        freq_layout.addLayout(bw_hbox)
+        freq_layout.addLayout(fstop_hbox)
+        freq_layout.addLayout(freq_inc_hbox)
+        freq_layout.addLayout(rbw_hbox)
+        freq_group.setLayout(freq_layout)
+        
+        return freq_group
+    def _center_freq(self):
         cfreq = QtGui.QPushButton('Center')
         cfreq.setToolTip("[2]\nTune the center frequency") 
         self._cfreq = cfreq
         cfreq.clicked.connect(lambda: cu._select_center_freq(self))
-        freq = QtGui.QLineEdit(str(self.plot_state.center_freq/constants.MHZ))
-        self._freq_edit = freq
+        freq_edit = QtGui.QLineEdit(str(self.plot_state.center_freq/constants.MHZ))
+        self._freq_edit = freq_edit
         self.control_widgets.append(self._cfreq)
         self.control_widgets.append(self._freq_edit)
         def freq_change():
             cu._select_center_freq(self)
             self.update_freq()
             self.update_freq_edit()
-        freq.returnPressed.connect(lambda: freq_change())
+        
+        freq_edit.returnPressed.connect(lambda: freq_change())
+        return cfreq, freq_edit
+    
+    def _freq_incr(self):
         steps = QtGui.QComboBox(self)
         steps.addItem("Adjust: 1 MHz")
         steps.addItem("Adjust: 2.5 MHz")
@@ -405,7 +383,7 @@ class MainPanel(QtGui.QWidget):
         self._fstep_box = steps
         def freq_step(factor):
             try:
-                f = float(freq.text())
+                f = float(self._freq_edit.text())
             except ValueError:
                 return
             delta = float(steps.currentText().split()[1]) * factor
@@ -420,7 +398,8 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._freq_plus)
         self.control_widgets.append(self._freq_minus)
         self.control_widgets.append(self._fstep_box)
-        return cfreq, freq, steps, freq_plus, freq_minus
+        return  steps, freq_plus, freq_minus
+    
     
     def _bw_controls(self):
         bw = QtGui.QPushButton('Span')
@@ -530,6 +509,79 @@ class MainPanel(QtGui.QWidget):
         self._freq_edit.setText("%0.1f" % (self.plot_state.center_freq / 1e6))
         self._bw_edit.setText("%0.1f" % (self.plot_state.bandwidth / 1e6))
     
+    def _plot_controls(self):
+
+        plot_group = QtGui.QGroupBox("Plot Control")
+        self._plot_group = plot_group
+        
+        plot_controls_layout = QtGui.QVBoxLayout()
+        
+        first_row = QtGui.QHBoxLayout()
+        first_row.addWidget(self._marker_control())
+        first_row.addWidget(self._delta_control())
+        
+        second_row = QtGui.QHBoxLayout()
+        second_row.addWidget(self._peak_control())
+        second_row.addWidget(self._mhold_control())
+        
+        third_row = QtGui.QHBoxLayout()
+        third_row.addWidget(self._pause_control())
+        third_row.addWidget(self._center_control())
+        
+        plot_controls_layout.addLayout(first_row)
+        plot_controls_layout.addLayout(second_row)
+        plot_controls_layout.addLayout(third_row)
+        
+        plot_group.setLayout(plot_controls_layout)
+        
+        return plot_group
+    def _marker_control(self):
+        marker = QtGui.QCheckBox('Marker 1')
+        marker.setToolTip("[M]\nTurn Marker 1 on/off") 
+        marker.clicked.connect(lambda: cu._marker_control(self))
+        self._marker = marker
+        self.control_widgets.append(self._marker)
+        return marker
+        
+    def _delta_control(self):
+        delta = QtGui.QCheckBox('Marker 2')
+        delta.setToolTip("[K]\nTurn Marker 2 on/off") 
+        delta.clicked.connect(lambda: cu._delta_control(self))
+        self._delta = delta
+        self.control_widgets.append(self._delta)
+        return delta
+    
+    def _peak_control(self):
+        peak = QtGui.QPushButton('Peak')
+        peak.setToolTip("[P]\nFind peak of the selected spectrum") 
+        peak.clicked.connect(lambda: cu._find_peak(self))
+        self._peak = peak
+        self.control_widgets.append(self._peak)
+        return peak
+        
+    def _mhold_control(self):
+        mhold = QtGui.QPushButton('Max Hold')
+        mhold.setToolTip("[H]\nTurn the Max Hold on/off") 
+        mhold.clicked.connect(lambda: cu._mhold_control(self))
+        self._mhold = mhold
+        self.control_widgets.append(self._mhold)
+        return mhold
+        
+    def _center_control(self):
+        center = QtGui.QPushButton('Recenter')
+        center.setToolTip("[C]\nCenter the Plot View around the available spectrum") 
+        center.clicked.connect(lambda: cu._center_plot_view(self))
+        self._center_bt = center
+        self.control_widgets.append(self._center_bt)
+        return center
+        
+    def _pause_control(self):
+        pause = QtGui.QPushButton('Pause')
+        pause.setToolTip("[Space Bar]\n pause the plot window") 
+        pause.clicked.connect(lambda: cu._enable_plot(self))
+        self._pause = pause
+        self.control_widgets.append(self._pause)
+        return pause
     def _marker_labels(self):
         marker_label = QtGui.QLabel('')
         marker_label.setStyleSheet('color: %s;' % constants.TEAL)
